SystemOrganization addCategory: #REPL!

!String methodsFor: '*REPL-printing' stamp: 'mvdg 7/7/2007 17:09'!
printMultiLineOn: aStream
	aStream nextPut: $'.
	aStream nextPutAll: (self withNoLineLongerThan: 70).
	aStream nextPut: $'.
! !

TcpService subclass: #REPLServer
	instanceVariableNames: 'password socketsToDestroy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!REPLServer commentStamp: 'mvdg 11/6/2006 21:25' prior: 0!
I'm a REPLServer - that is, a Read-Eval-Print loop network server.
Do:

REPLServer startOn: 4445.

This will make me start on port 4445. Grab your telnet client and telnet to localhost, port 4445.
!

!REPLServer class methodsFor: 'initialization' stamp: 'mvdg 7/15/2007 20:26'!
start
	^ self startOn: 4445 named: 'REPLServer' priority: (Processor lowIOPriority).! !

!REPLServer class methodsFor: 'initialization' stamp: 'mvdg 7/15/2007 20:54'!
stop
	self allServices do: [ :each | (each isKindOf: self) ifTrue: [each unregister] ]! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 22:10'!
cleanSocketList
	socketsToDestroy := socketsToDestroy select: [ :each | each isConnected ].! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 22:30'!
destroySockets
	"Mwa ha ha ha ha."
	socketsToDestroy do: [ :each | 
		each isConnected ifTrue: [each close. each destroy ] ].
	socketsToDestroy := Bag new.! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 22:19'!
getPasswordFrom: aStream
	| rv |
	aStream nextPutAll: 'Password: '; flush.
	rv := aStream nextLine = self password.
	aStream crlf.
	^ rv.! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 22:18'!
initialize
	socketsToDestroy := Bag new.
	^ super initialize.! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 22:24'!
password
	^ password.! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 22:15'!
prepareForShutdown
	self destroySockets.
	^ super prepareForShutdown.! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 22:09'!
serve: aSocket
	| stream socketStream |
	self cleanSocketList.
	socketsToDestroy add: aSocket.
	(aSocket remoteAddress) = (aSocket localAddress) ifFalse: [ aSocket close ].
	socketStream := SocketStream on: aSocket.
	socketStream binary.
	socketStream noTimeout.
	stream := (ANSIStream new) sourceStream: (TelnetStream new sourceStream: socketStream).
	(self getPasswordFrom: stream)
		ifTrue: [REPL startOn: stream]
		ifFalse: [stream nextPutAll: 'Access denied.'; cr].
	stream close.
! !

!REPLServer methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 22:28'!
verifyOkToStart
	| tempPassword |
	password isNil ifTrue: [ 
		[tempPassword := FillInTheBlank request: 'Please choose a new REPLServer password: '.
		 password := FillInTheBlank request: 'Please type it in again: '.
		 tempPassword = password]
			whileFalse: [ self inform: 'Those passwords did not match.' ]].
	^ super verifyOkToStart.! !

!Collection methodsFor: '*REPL-printing' stamp: 'mvdg 2/6/2007 12:05'!
printMultiLineOn: aStream
	self printNameOn: aStream.
	aStream nextPut: $(.
	self do: [:element | aStream crtab; print: element].
	aStream cr;nextPut: $).! !

OrderedCollection subclass: #ProcessList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!ProcessList commentStamp: 'mvdg 11/20/2006 20:31' prior: 0!
I am a list of processes which the user can interact with. I have methods to pretty-print processes.!

!ProcessList methodsFor: 'as yet unclassified' stamp: 'mvdg 7/15/2007 20:37'!
printMultiLineOn: aStream
	| count |
	count := 1.
	aStream nextPut: $(.
	self do: [ :element | 
		aStream crtab; 
		print: count; nextPut: $:; space;
		nextPutAll: element name; nextPutAll: ': ';
		nextPutAll: element browserPrintString.
		count := count + 1.].
	aStream cr;nextPut: $)! !

!Text methodsFor: '*REPL-printing' stamp: 'mvdg 7/13/2007 21:25'!
printMultiLineOn: aStream
	" We could be fancy here and do bold, colour etc... "
	aStream nextPutAll: string.! !

!SequenceableCollection methodsFor: '*REPL-printing' stamp: 'mvdg 7/13/2007 21:27'!
printMultiLineOn: aStream
	| count |
	count := 1.
	aStream nextPut: $(.
	self do: [ :element | 
		aStream crtab; 
		print: count; nextPut: $:; space;
		nextPutAll: ((element printStringLimitedTo: 54) select: [ :each | each isTypeable ]).
		count := count + 1.].
	aStream cr;nextPut: $)! !

Object subclass: #ConnectingStream
	instanceVariableNames: 'sourceStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

ConnectingStream subclass: #ANSIStream
	instanceVariableNames: 'csiMap specialChars ansiToChar parseNumberDict parseCursorKeys parseESCDict parseNumPadOffDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!ANSIStream commentStamp: 'mvdg 11/12/2006 14:02' prior: 0!
I convert Squeak characters into ANSI characters.

Usage:

s := ANSIStream << (TelnetStream on: Socket).

"s next" will return a Character.
"s nextPut: aChar" will write that Character (esp if it is "Character pageUp" or similar) to the TelnetStream.

My interface handles Characters, but my sourceStream must handle Integers.!

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 15:58'!
arrowDown
	sourceStream nextPutAll: { 27. 91. 66. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 15:58'!
arrowLeft
	sourceStream nextPutAll: { 27. 91. 68. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 15:58'!
arrowRight
	sourceStream nextPutAll: { 27. 91. 67. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 15:58'!
arrowUp
	sourceStream nextPutAll: { 27. 91. 65. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 16:57'!
backspace
	" I'm having trouble finding a code to do this... "
	sourceStream nextPutAll: { 8. 27. 91. 49. 80. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:45'!
black
	sourceStream nextPutAll: { 27. 91. 51. 48. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
blackBackground
	sourceStream nextPutAll: { 27. 91. 52. 48. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:45'!
blink
	sourceStream nextPutAll: { 27. 91. 53. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
blue
	sourceStream nextPutAll: { 27. 91. 51. 52. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
blueBackground
	sourceStream nextPutAll: { 27. 91. 52. 52. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:42'!
bold
	sourceStream nextPutAll: { 27. 91. 49. 109. }.! !

!ANSIStream methodsFor: 'streaming' stamp: 'mvdg 11/14/2006 20:51'!
clearLine
	" Clear the current screen line. Kind of like cr and arrowUp."
	sourceStream nextPutAll: { 27. 91. 50. 75. 13. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:45'!
concealed
	sourceStream nextPutAll: { 27. 91. 56. 109. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 7/13/2007 20:38'!
cr
	" Here we assume that cr returns the cursor to the start of the next line down. "
	^ self crlf.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 14:51'!
crlf
	sourceStream nextPut: (Character cr); nextPut: (Character lf).! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:37'!
crossedOut
	" With a horizontal line through the text. "
	sourceStream nextPutAll: { 27. 91. 57. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:47'!
cyan
	sourceStream nextPutAll: { 27. 91. 51. 54. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
cyanBackground
	sourceStream nextPutAll: { 27. 91. 52. 54. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:42'!
encircled
	sourceStream nextPutAll: { 27. 91. 53. 50. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:44'!
faint
	sourceStream nextPutAll: { 27. 91. 50. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:39'!
font1
	" The default font. "
	sourceStream nextPutAll: { 27. 91. 49. 48. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:40'!
font10
	sourceStream nextPutAll: { 27. 91. 49. 57. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:39'!
font2
	sourceStream nextPutAll: { 27. 91. 49. 49. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:39'!
font3
	sourceStream nextPutAll: { 27. 91. 49. 50. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:39'!
font4
	sourceStream nextPutAll: { 27. 91. 49. 51. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:39'!
font5
	sourceStream nextPutAll: { 27. 91. 49. 52. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:40'!
font6
	sourceStream nextPutAll: { 27. 91. 49. 53. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:40'!
font7
	sourceStream nextPutAll: { 27. 91. 49. 54. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:40'!
font8
	sourceStream nextPutAll: { 27. 91. 49. 55. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:40'!
font9
	sourceStream nextPutAll: { 27. 91. 49. 56. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:41'!
fractur
	" The spec says this is a Gothic font!! :-) "
	sourceStream nextPutAll: { 27. 91. 50. 48. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:42'!
framed
	sourceStream nextPutAll: { 27. 91. 53. 49. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:46'!
green
	sourceStream nextPutAll: { 27. 91. 51. 50. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
greenBackground
	sourceStream nextPutAll: { 27. 91. 52. 50. 109. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/14/2006 20:34'!
insertCharacter
	sourceStream nextPutAll: { 27. 91. 64. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:44'!
italic
	sourceStream nextPutAll: { 27. 91. 51. 109. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/14/2006 22:29'!
lf
	sourceStream nextPut: 10.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:46'!
magenta
	sourceStream nextPutAll: { 27. 91. 51. 53. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:34'!
magentaBackground
	sourceStream nextPutAll: { 27. 91. 52. 53. 109. }.! !

!ANSIStream methodsFor: 'characters' stamp: 'mvdg 11/12/2006 16:42'!
newPage
	" broken... "
	sourceStream nextPutAll: { 27. 91. 50. 74. 27. 91. 72. }.! !

!ANSIStream methodsFor: 'streaming' stamp: 'mvdg 7/8/2007 17:17'!
next
	| v |
	ansiToChar ifNil: [ ansiToChar := Dictionary newFrom: {
		" I made up most of these based on Linux command-line stuff, Feel free to disagree. "
		" I'd like to make CTRL-x,c,v -> cut,copy,paste "
		1 -> [ Character home ]. "CTRL-a through CRTL-z"
		2 -> [ Character arrowLeft ].
		3 -> [ Character interrupt ].
		4 -> [ Character delete ].
		5 -> [ Character end ].
		6 -> [ Character arrowRight ].
		7 -> [ Character beep ].
		8 -> [ Character backspace ]. "CTRL-backspace"
		9 -> [ Character tab ].
		10 -> [ Character lf ].
		11 -> [ Character pageUp ]. " ?? in Unix, this cuts the line. "
		12 -> [ Character newPage ].
		13 -> [ Character cr ].
		14 -> [ Character arrowDown ].
		15 -> [ $? ]. "Dunno... CTRL-o? "
		16 -> [ Character arrowUp ].
		17 -> [ Character continue ].
		18 -> [ $? ].
		19 -> [ Character suspend ].
		20 -> [ $? ].
		21 -> [ $? ].
		22 -> [ Character pageDown ].
		23 -> [ $? ].
		24 -> [ $? ].
		25 -> [ $? ].
		26 -> [ Character suspend ].
		
		27 -> [ self parseESC ].
		127 -> [ Character backspace ].
	} ].

	v := sourceStream next.
	v ifNil: [ ^ nil ].
	^ (ansiToChar at: v ifAbsent: [ [v asCharacter] ]) value.

! !

!ANSIStream methodsFor: 'streaming' stamp: 'mvdg 7/8/2007 16:14'!
nextPut: aCharacter
	specialChars ifNil: [ specialChars := Dictionary newFrom: {
		(Character arrowUp) -> [ self arrowUp ].
		(Character arrowDown) -> [ self arrowDown ].
		(Character arrowLeft) -> [ self arrowLeft ].
		(Character arrowRight) -> [ self arrowRight].
		(Character newPage) -> [ self newPage ].
		(Character backspace) -> [ self backspace ].
		(Character cr) -> [self crlf].
		(Character lf) -> [].
	} ]. " idea: I could add Colors here too!! "
	(specialChars includesKey: aCharacter) 
		ifTrue: [(specialChars at: aCharacter) value]
		ifFalse: [ self insertCharacter. sourceStream nextPut: (aCharacter asInteger) ].! !

!ANSIStream methodsFor: 'streaming' stamp: 'mvdg 11/14/2006 21:49'!
nextPutAll: aString
	aString do: [ :each | self nextPut: each ].! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:43'!
normal
	sourceStream nextPutAll: { 27. 91. 48. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/16/2006 21:42'!
overlined
	sourceStream nextPutAll: { 27. 91. 53. 51. 109. }.! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 7/8/2007 17:22'!
parseCSI: argSoFar
	| v |
	csiMap ifNil: [ csiMap := Dictionary newFrom: {
		48 -> [ :arg | self parseCSI: 10*arg ].
		49 -> [ :arg | self parseCSI: 10*arg+1 ].
		50 -> [ :arg | self parseCSI: 10*arg+2 ].
		51 -> [ :arg | self parseCSI: 10*arg+3 ].
		52 -> [ :arg | self parseCSI: 10*arg+4 ].
		53 -> [ :arg | self parseCSI: 10*arg+5 ].
		54 -> [ :arg | self parseCSI: 10*arg+6 ].
		55 -> [ :arg | self parseCSI: 10*arg+7 ].
		56 -> [ :arg | self parseCSI: 10*arg+8 ].
		57 -> [ :arg | self parseCSI: 10*arg+9 ].
		65 -> [ :arg |  Character arrowUp ].
		66 -> [ :arg |  Character arrowDown ].
		67 -> [ :arg |  Character arrowRight ].
		68 -> [ :arg |  Character arrowLeft ].
		70 -> [ :arg |  Character end ].
		72 -> [ :arg |  Character home ].
		97 -> [ :arg |  Character arrowUp ]. "with shift. "
		98 -> [ :arg |  Character arrowDown ]. "with shift. "
		99 -> [ :arg |  Character arrowRight ]. "with shift. "
		100 -> [ :arg |  Character arrowLeft ]. "with shift. "
		94 -> [ :arg | self parseUnknownCode: arg ].
		126 -> [ :arg | self parseEditKeys: arg ].
	} ].

	(argSoFar > 1000) ifTrue: [ ^ $? ]. "Somebody is playing with us."
	v := sourceStream next.
	" This is really dumb. The 'copy' is put there to prevent an error when you press F1. Try it... "
	^ (csiMap at: v ifAbsent: [ [:arg | "self halt." $?] ]) copy value: argSoFar.! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 11/16/2006 23:19'!
parseESC
	| v |
	parseESCDict ifNil: [ parseESCDict := Dictionary newFrom: {
		91 -> [ self parseCSI: 0 ].
		79 -> [ self parseNumPadOff ].
	}].

	v := sourceStream next.
	^ (parseESCDict at: v ifAbsent: [[self parseEscapedChar: v]]) value.	
! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 7/8/2007 17:12'!
parseEditKeys: aNumber
	" Usually a number is an argument for one of the following: "
	parseCursorKeys ifNil: [ parseCursorKeys := Dictionary newFrom: {
		2 -> Character insert.
		3 -> Character delete.
		5 -> Character pageUp.
		6 -> Character pageDown.
		7 -> Character home.
		8 -> Character end.
		12 -> $?. " F1 "
		13 -> $?. " F2 "
		14 -> $?. " F3 "
		15 -> $?. " F4 "
		16 -> $?. " F5 "
		17 -> $?. " F6 "
		18 -> $?. " F7 "
		19 -> $?. " F8 "
		20 -> $?. " F9 "
		21 -> $?. " F10 "
		22 -> $?. " F11 or shift-F1"
		23 -> $?. " F12 or shift-F2"
		24 -> $?. " shift-F3 "
		25 -> $?. " shift-F4 "
		26 -> $?. " shift-F5 "
		27 -> $?. " shift-F6 "
		28 -> $?. " shift-F7 "
		29 -> $?. " shift-F8 "
		30 -> $?. " shift-F9 "
		31 -> $?. " shift-F10 "
		32 -> $?. " shift-F11 "
		33 -> $?. " shift-F12 "
		
	} ].
	^ parseCursorKeys at: aNumber ifAbsent: [ $? ].! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 7/8/2007 17:09'!
parseEscapedChar: v
	" The user is holding down 'ALT'. "
	| c |
	c := sourceStream next.
	" I don't really know what to do here. How can I tell my user that the user is holding down ALT?"
	c isNil ifTrue: [ ^ nil ] ifFalse: [ ^ c asCharacter ].! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 11/16/2006 23:27'!
parseNumPadOff
	parseNumPadOffDict ifNil: [ parseNumPadOffDict := Dictionary newFrom: {
		77 -> Character cr.
		97 -> Character arrowUp. "With CTRL"
		98 -> Character arrowDown. "With CTRL"
		99 -> Character arrowRight. "With CTRL"
		100 -> Character arrowLeft. "With CTRL "
		106 -> $*.
		107 -> $+.
		109 -> $-.
		110 -> Character delete.
		111 -> $/.
		112 -> Character insert.
		113 -> Character end.
		114 -> Character arrowDown.
		115 -> Character pageDown.
		116 -> Character arrowLeft.
		117 -> $+. "Center, 5. "
		118 -> Character arrowRight.
		119 -> Character home.
		120 -> Character arrowUp.
		121 -> Character pageUp.
		}].
	^ (parseNumPadOffDict at: (sourceStream next) ifAbsent: [[$?]]) value.! !

!ANSIStream methodsFor: 'parsing' stamp: 'mvdg 11/16/2006 23:30'!
parseUnknownCode: arg
	" This happens when you do CTRL-<key> for special keys. "
	^ $?! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:45'!
rapidBlink
	sourceStream nextPutAll: { 27. 91. 54. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:46'!
red
	sourceStream nextPutAll: { 27. 91. 51. 49. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:35'!
redBackground
	sourceStream nextPutAll: { 27. 91. 52. 49. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:45'!
reverse
	sourceStream nextPutAll: { 27. 91. 55. 109. }.! !

!ANSIStream methodsFor: 'styles' stamp: 'mvdg 11/12/2006 15:44'!
underlined
	sourceStream nextPutAll: { 27. 91. 52. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:49'!
white
	sourceStream nextPutAll: { 27. 91. 51. 55. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:37'!
whiteBackground
	sourceStream nextPutAll: { 27. 91. 52. 55. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/12/2006 15:46'!
yellow
	sourceStream nextPutAll: { 27. 91. 51. 51. 109. }.! !

!ANSIStream methodsFor: 'colors' stamp: 'mvdg 11/16/2006 21:35'!
yellowBackground
	sourceStream nextPutAll: { 27. 91. 52. 51. 109. }.! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 21:48'!
close
	^ [sourceStream close] ifError: [nil].! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 21:28'!
flush
	^ [sourceStream flush] ifError: [nil].! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/12/2006 13:49'!
next
	^ sourceStream next.! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/14/2006 23:42'!
nextLine
	| o n keepGoing c |
	o := WriteStream on: String new.
	keepGoing := true.
	[keepGoing] whileTrue: [ 
		n := sourceStream next.
		n ifNil: [ ^ '' ].
		c := n asCharacter.
		(c = Character cr) ifTrue: [ keepGoing := false ]
			ifFalse: [ o nextPut: c. ].
	].
	^ o contents.! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/12/2006 13:49'!
nextPut: something
	^ sourceStream nextPut: something.! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/12/2006 13:49'!
nextPutAll: aCollection
	^ sourceStream nextPutAll: aCollection.! !

!ConnectingStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/11/2006 23:06'!
sourceStream: aStream
	sourceStream := aStream.
! !

ConnectingStream subclass: #TelnetStream
	instanceVariableNames: 'iacCommands optionMode optionsSwitch cr lf'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!TelnetStream commentStamp: 'mvdg 11/12/2006 14:01' prior: 0!
I convert a stream of RFC854 Telnet commands into a stream of usable characters. These characters will still likely need to be passed through an ANSIStream to be useful.

Both my input and my output are Integers. Use a ANSIStream to convert this to characters.!

!TelnetStream methodsFor: 'control' stamp: 'mvdg 11/11/2006 21:19'!
do
	self nextPut: 255; nextPut: 253.
	^ self.! !

!TelnetStream methodsFor: 'control' stamp: 'mvdg 11/11/2006 21:20'!
dont
	self nextPut: 255; nextPut: 254.
	^ self.! !

!TelnetStream methodsFor: 'telnet options' stamp: 'mvdg 11/11/2006 23:08'!
dontEcho
	" The sender of this command REQUESTS to begin, or confirms that it
      will now begin, echoing data characters it receives over the
      TELNET connection back to the sender of the data characters. "
	self dont; nextPut: 1.
	! !

!TelnetStream methodsFor: 'telnet options' stamp: 'mvdg 11/11/2006 21:53'!
dontLinemode
	self dont; nextPut: 34.! !

!TelnetStream methodsFor: 'stream' stamp: 'mvdg 7/17/2007 21:18'!
next
	| v int keepGoing |
	keepGoing := true.
	[keepGoing] whileTrue: [ "Loop on multi-byte control codes"
		[ v := sourceStream next ]
			ifError: [ :error :receiver |
				" Assume the Socket has died. "
				[sourceStream close] ifError: [].
				^ nil ].
		v isNil ifTrue: [ ^ nil. ]
			ifFalse: [
				int := v asInteger.
				(v = 255) ifTrue: [ int := self parseIAC ].
				int ifNotNil: [ keepGoing := false ].
		].
	].
	self assert: [ int isKindOf: Integer ].
	^ int.! !

!TelnetStream methodsFor: 'initialize-release' stamp: 'mvdg 11/11/2006 22:36'!
optionsSwitch
	optionsSwitch ifNil: [ optionsSwitch := Dictionary newFrom: {
		0 -> [ self parseOptionTransmitBinary ].
		1 -> [ self parseOptionEcho ].
		3 -> [ self parseOptionSuppressGoAhead ].
		5 -> [ self parseOptionStatus ].
		6 -> [ self parseOptionTimingMark ].
		31 -> [ self parseOptionNegotiateWindowSize ].
		34 -> [ self parseOptionLineMode ].
		255 -> [ self parseOptionExtendedOptionsList ].
	} ].
	^ optionsSwitch. ! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:27'!
parseAbortOutput
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:27'!
parseAreYouThere
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:27'!
parseBreak
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:27'!
parseDataMark
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/12/2006 13:56'!
parseDo
	| v |
	optionMode := #do.
	v := sourceStream next.
	v ifNil: [ self halt: 'Got nil over the stream. ' ].
	v := v asInteger.
	^ ((self optionsSwitch) at: v ifAbsent: [ [nil] ]) value.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/12/2006 13:57'!
parseDont
	| v |
	optionMode := #dont.
	v := sourceStream next.
	^ ((self optionsSwitch) at: v ifAbsent: [ [nil] ]) value.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:22'!
parseDoubleIAC
	^ 255.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:27'!
parseEndSubnegotiation
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:28'!
parseEraseCharacter
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/15/2006 08:28'!
parseEraseLine
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:33'!
parseGoAhead
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/12/2006 14:18'!
parseIAC
	| nextByte |
	iacCommands ifNil: [ iacCommands := Dictionary newFrom: {
		240 -> [ self parseEndSubnegotiation ].
		241 -> [ self parseNOP ].
		242 -> [ self parseDataMark ].
		243 -> [ self parseBreak ].
		244 -> [ self parseInterruptProcess ].
		245 -> [ self parseAbortOutput ].
		246 -> [ self parseAreYouThere ].
		247 -> [ self parseEraseCharacter ].
		248 -> [ self parseEraseLine ].
		249 -> [ self parseGoAhead ].
		250 -> [ self parseSubnegotiation ].
		251 -> [ self parseWill ].
		252 -> [ self parseWont ].
		253 -> [ self parseDo ].
		254 -> [ self parseDont ].
		255 -> [ self parseDoubleIAC ].
	} ].
	nextByte := sourceStream next.
	nextByte ifNil: [ self halt: 'Got nil over a stream'].
	nextByte := nextByte asInteger.
	^ (iacCommands at: nextByte ifAbsent: [ [nil] ]) value.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:33'!
parseInterruptProcess
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:33'!
parseNOP
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:33'!
parseOptionEcho
	" Not implemented "
	^ nil. ! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:35'!
parseOptionExtendedOptionList
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionLineMode
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionNegotiateWindowSize
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionStatus
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionSuppressGoAhead
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionTimingMark
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseOptionTransmitBinary	
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/11/2006 22:34'!
parseSubnegotiation
	" Not implemented "
	^ nil.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/12/2006 13:57'!
parseWill
	| v |
	optionMode := #will.
	v := sourceStream next.
	^ ((self optionsSwitch) at: v ifAbsent: [ [nil] ]) value.! !

!TelnetStream methodsFor: 'parsing' stamp: 'mvdg 11/12/2006 13:57'!
parseWont
	| v |
	optionMode := #wont.
	v := sourceStream next.
	^ ((self optionsSwitch) at: v ifAbsent: [ [nil] ]) value.! !

!TelnetStream methodsFor: 'initialize-release' stamp: 'mvdg 7/7/2007 17:30'!
sourceStream: aStream
	| rv |
	cr := Character cr asInteger.
	lf := Character lf asInteger.
	rv := super sourceStream: aStream.
	self dontLinemode.
	self willEcho.
	self dontEcho.
	self willSuppressGoAhead.
	^ rv.
! !

!TelnetStream methodsFor: 'control' stamp: 'mvdg 11/11/2006 21:19'!
will
	self nextPut: 255; nextPut: 251.
	^ self.! !

!TelnetStream methodsFor: 'telnet options' stamp: 'mvdg 11/11/2006 21:31'!
willEcho
	" The sender of this command REQUESTS to begin, or confirms that it
      will now begin, echoing data characters it receives over the
      TELNET connection back to the sender of the data characters. "
	self will; nextPut: 1.
	! !

!TelnetStream methodsFor: 'telnet options' stamp: 'mvdg 11/11/2006 23:22'!
willSuppressGoAhead
	self will; nextPut: 3.! !

!TelnetStream methodsFor: 'control' stamp: 'mvdg 11/11/2006 21:19'!
wont
	self nextPut: 255; nextPut: 252.
	^ self.! !

Object subclass: #Help
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/18/2007 21:54'!
bugs
	^ '
---------------------------------------------------------------------
Known bugs and wanted features

- There''s no way to see which process is eating all the CPU. It should be possible to see CPU %usage per process.

- REPLServer becomes unresponsive when Squeak is under load, even at high priorities.

- It would be nice to have a Canvas-based REPLServer (i.e. when there''s a GUI) as well as the telnet version.

- Text could be printed using colours and fonts :-).

- Currently, ''yes | netcat localhost 4445'' will crash REPLServer, even at login.'! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 21:14'!
classes
 ^ '
---------------------------------------------------------------------
Here are some useful things you can do with classes:

''anObject class'' -- Returns that object''s class.
''aClass comment'' -- Shows the documentation for that Class.
''aClass comment asString withNoLineLongerThan: 70'' -- same, but nicer output.
''aClass selectors'' -- Shows the methods this Class has.
''aClass allSelectors'' -- Shows the methods of all superclasses as well.
''aClass allInstances'' -- Shows all instances of this Class.
''aClass sourceMethodAt: #selectorName'' -- Show method source.

Type ''help processes'' to continue the tutorial.
'.
		! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 21:12'!
editing
 ^ '
---------------------------------------------------------------------
You can do basic command-line editing. REPLServer assumes you are using a VT100 or ANSI capable terminal.

You can use the up/down arrow keys to cycle through your previous commands. You can use left/right to move the cursor along your current line.

The ''home'' and ''end'' keys bring you to the start and end of your line. CTRL-a should also bring you to the beginning of your line, and CTRL-e should bring you to the end of your line.

Currently, multi-line editing is not yet supported.

Type ''help classes'' for the next tutorial page.
'.! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 20:44'!
help
	^ '
---------------------------------------------------------------------
Welcome to the Read-Eval-Print Loop telnet server.

I evaluate Smalltalk expressions and print the result.

Type ''help tutorial.'' for a tutorial on REPL.
Type ''help class selectors'' for a list of all help commands.
Type ''bindings'' to list all current variable bindings.
Type ''exit'' to quit.
'.! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/15/2007 21:06'!
image
	^ '
---------------------------------------------------------------------

To quit Squeak, saving the image:
''utils quitSqueak.''

To do it the wordier way:
''SmalltalkImage current snapshot: true andQuit: true.

Should that fail:
''SmalltalkImage current snapShotPrimitive.''
(that returns nil if it fails).
''SmalltalkImage current quitPrimitive.''


'! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/8/2007 16:56'!
last
 ^ '
---------------------------------------------------------------------
The result of every expression you type in is assigned to the variable ''last'' and printed on your telnet session. For example, if after the previous command, you type: 

: ''dehllloorw''
(...)
: last size.
11

You will see that the String ''dehllloorw'' is 11 Characters long. If you then type ''last'' again, you will see that ''last'' has now been assigned to the Integer 11.

If you want to keep the result of a command, then you can bind it to another variable. For example:

: Process allInstances select: [ :each | each priority > 40 ].
(...)
: highProcesses := last.

Type ''help editing'' to continue.
'.! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 20:48'!
printOn: aStream
 	aStream nextPutAll: 'a Help(type ''help help.'' for help)'.! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/15/2007 20:57'!
processes
	^ '
---------------------------------------------------------------------
Some useful commands on a Process:

''utils allProcesses'' -- list all non-terminated Processes.
''(last at: 4) suspend'' -- suspend number 4 above.
''utils suspendAllProcesses'' -- suspend all processes at user priority or below.
''aProcess suspend'' -- suspend that particular process.
''aProcess resume'' -- resume that particular process.
''aProcess terminate'' -- kill that process.
''aProcess debug'' -- Pop up a Debugger in Morphic with that Process.

Also, to shut down Squeak:
''Smalltalk snapshot: true andQuit: true.''

The next page of the tutorial can be seen by doing ''help utils''.
'! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/8/2007 16:56'!
tutorial
	^ '
---------------------------------------------------------------------
Welcome to the Read-Eval-Print Loop telnet server.

This is a Smalltalk interpreter. You can type in any Smalltalk command and I will evaluate it. For example, if you type:

: helloWorld := ''hello world'' sort.
''dehllloorw''

Then the variable ''helloWorld'' will be assigned the value ''dehllloorw'' (i.e. all the Characters of ''hello world'' in alphabetical order). Variable bindings are made automatically but are lost when you close the connection.

Type ''help last'' to continue.

'.! !

!Help methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 21:11'!
utils
	^ '
---------------------------------------------------------------------
You will have noticed that there is a variable called ''utils'' available to you. If you do:

''utils class selectors.''

Then you will see that it (currently) has three methods: #suspendAllProcesses, #allProcesses and #rebootMorphic.

''utils suspendAllProcesses'' -- This will suspend all non-critical processes in Squeak.

''utils allProcesses'' -- This lists all active processes.

''utils rebootMorphic'' -- This resets Morphic to a new Project. This is useful when the Morphic UI has become unusable.

This is currently the end of the tutorial.
'! !

Object subclass: #InteractiveLineStream
	instanceVariableNames: 'sourceStream hinter history charActions lineComplete currentHistoryItem currentLine cursorPosition prompt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!InteractiveLineStream commentStamp: 'mvdg 11/7/2006 21:13' prior: 0!
I am a stream which is intended to be interacted with manually. I read one line at a time and allow the user to edit that line interactively.!

!InteractiveLineStream class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/7/2006 21:16'!
on: aStream withHinter: aHinter
	^ self new on: aStream withHinter: aHinter.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/19/2006 21:17'!
backspace
	(cursorPosition > 1) ifTrue: [ 
		cursorPosition := cursorPosition - 1.
		currentLine removeAt: cursorPosition.
		sourceStream backspace; flush.
	].! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/14/2006 22:23'!
clearScreen
	sourceStream newPage.
	self endOfLine.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 7/13/2007 22:01'!
cr
	sourceStream cr.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 2/6/2007 12:01'!
crtab
	^ self crtab: 1.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 2/6/2007 12:15'!
crtab: anInteger
	self cr.
	anInteger timesRepeat: [ self nextPut: Character tab].! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 11/14/2006 22:25'!
currentLineAsString
	| rv size |
	size := currentLine size.
	rv := String new: size.
	1 to: size do: [ :i | rv at: i put: (currentLine at: i) ].
	^ rv.! !

!InteractiveLineStream methodsFor: 'history' stamp: 'mvdg 11/12/2006 21:56'!
downHistory
	currentHistoryItem := (currentHistoryItem + 1) min: (history size + 1).
	currentHistoryItem > (history size) ifTrue: [
		" If the user scrolls down past the last history item, give them a fresh prompt. "
		self selectLine: (OrderedCollection new).
	] ifFalse: [
		self selectLine: (history at: currentHistoryItem).
	].! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/19/2006 21:40'!
endOfLine
	" This is also good for 'refreshing the screen' "
	sourceStream clearLine; nextPutAll: (self prompt); nextPutAll: (self currentLineAsString); flush.
	cursorPosition := (currentLine size + 1) max: 1.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 11/8/2006 07:55'!
flush
	^ sourceStream flush.! !

!InteractiveLineStream methodsFor: 'initialization' stamp: 'mvdg 11/23/2006 21:38'!
initCharActions
	charActions := Dictionary newFrom: {
		(Character cr) -> [ :c | self lineComplete ].
		(Character arrowUp) -> [ :c | self upHistory ].
		(Character arrowDown) -> [ :c | self downHistory ].
		(Character arrowLeft) -> [ :c | self leftChar ].
		(Character arrowRight) -> [ :c | self rightChar ].
		(Character backspace) -> [ :c | self backspace ].
		(Character home) -> [ :c | self startOfLine ].
		(Character end) -> [ :c | self endOfLine ].
		(Character newPage) -> [ :c | self clearScreen ].
		(Character tab) -> [ :c | self tabComplete ].
	}.! !

!InteractiveLineStream methodsFor: 'initialization' stamp: 'mvdg 11/14/2006 23:03'!
initialize
	history := OrderedCollection new.	
	currentHistoryItem := 1.
	lineComplete := false.
	prompt := '> '.
	self initCharActions.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/14/2006 19:55'!
leftChar
	(cursorPosition > 1) ifTrue: [
		cursorPosition := cursorPosition - 1.
		sourceStream arrowLeft; flush.
	].! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/9/2006 21:39'!
lineComplete
	lineComplete := true.
	sourceStream crlf.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 11/8/2006 07:39'!
next
	^ self shouldNotImplement.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 7/8/2007 17:10'!
nextLine
	| nextChar rv |
	lineComplete := false.
	currentLine := OrderedCollection new.
	cursorPosition := 1. 
	sourceStream nextPutAll: (self prompt); flush.

	[lineComplete] whileFalse: [
		nextChar := sourceStream next.
		nextChar ifNil: [ ^ nil ].
		(charActions includesKey: nextChar) 
		ifTrue: [ (charActions at: nextChar) value: nextChar ]
		ifFalse: [ self writeCharacter: nextChar ].
	].	

	rv := self currentLineAsString.
	history add: currentLine.
	currentHistoryItem := history size + 1.
	^ rv.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 11/8/2006 07:39'!
nextPut: aCharacter
	^ sourceStream nextPut: aCharacter.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 7/13/2007 21:59'!
nextPutAll: aString
	^ sourceStream nextPutAll: aString.
! !

!InteractiveLineStream methodsFor: 'initialization' stamp: 'mvdg 11/14/2006 20:42'!
on: aStream withHinter: aHinter
	sourceStream := aStream.
	hinter := aHinter.
	! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 2/6/2007 12:06'!
print: anything
	anything printOn: self.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/14/2006 23:03'!
prompt
	^ prompt.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/14/2006 23:03'!
prompt: aPrompt
	self assert: [ aPrompt isKindOf: String ].
	prompt := aPrompt.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/23/2006 21:39'!
redraw
	sourceStream clearLine; nextPutAll: (self prompt).
	cursorPosition := currentLine size + 1.
	sourceStream nextPutAll: currentLine.
	sourceStream flush.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 11/14/2006 19:57'!
rightChar
	(cursorPosition <= currentLine size) ifTrue: [
		cursorPosition := cursorPosition + 1.
		sourceStream arrowRight; flush.
	].! !

!InteractiveLineStream methodsFor: 'history' stamp: 'mvdg 11/23/2006 21:40'!
selectLine: newLine
	self assert: [ newLine isKindOf: OrderedCollection ].
	currentLine := newLine copy.
	self redraw.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 2/6/2007 12:05'!
skip: anything
	self halt.! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 7/13/2007 21:28'!
space
	sourceStream nextPut: $ .! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 7/15/2007 20:51'!
startOfLine
	cursorPosition - 1 timesRepeat: [ sourceStream arrowLeft ].
	sourceStream flush.
	cursorPosition := 1.! !

!InteractiveLineStream methodsFor: 'line editing' stamp: 'mvdg 7/17/2007 21:45'!
tabComplete
	"currentLine := OrderedCollection newFrom: (hinter completeLine: (String newFrom: currentLine))."
	self redraw.! !

!InteractiveLineStream methodsFor: 'history' stamp: 'mvdg 11/14/2006 23:43'!
upHistory
	currentHistoryItem := (currentHistoryItem - 1) max: 1.
	(history size >= currentHistoryItem) ifTrue: [ 
		self selectLine: (history at: currentHistoryItem). ].! !

!InteractiveLineStream methodsFor: 'stream handling' stamp: 'mvdg 11/14/2006 21:06'!
writeCharacter: nextChar
	nextChar isTypeable ifTrue: [ 
		(cursorPosition = (currentLine size + 1)) ifTrue: [
			" OrderedCollection>>add:afterIndex: does not seem to work in this case."
			currentLine addLast: nextChar.
		] ifFalse: [
			currentLine add: nextChar afterIndex: cursorPosition - 1.
		].
		cursorPosition := cursorPosition + 1.
		sourceStream nextPut: nextChar.
		sourceStream flush.
	].! !

!Object methodsFor: '*repl-printing' stamp: 'mvdg 2/6/2007 11:46'!
printMultiLineOn: aStream
	^ self printOn: aStream.! !

Object subclass: #REPL
	instanceVariableNames: 'stream bindings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!REPL commentStamp: 'mvdg 11/6/2006 21:27' prior: 0!
I'm a Read-Eval-Print loop thingie. If you do:

REPL startOn: aStream.

then I'll listen to that stream, executing the commands that I receive on it. I'm intended to be interactive.!

!REPL class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/23/2006 22:03'!
on: aStream
	| me hinter |
	me := self basicNew.
	hinter := REPLHinter new compilerTarget: me.
	^ me initializeWithStream: (InteractiveLineStream on: aStream withHinter: hinter).! !

!REPL class methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 20:46'!
printBannerOn: aStream
	"[ "
	aStream crlf; nextPutAll: '  (\   /)     ';crlf.
	aStream nextPutAll: '    @ @       '.
	SystemVersion current printOn: aStream.
	aStream crlf; nextPutAll: '  == o ==            Type ''help help'' for help. ';crlf; crlf.
	"] ifError: []."! !

!REPL class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2006 21:52'!
startOn: aStream
	self printBannerOn: aStream.
	(self on: aStream) start! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 7/13/2007 21:34'!
bind: key to: value
	bindings ifNil: [ self initializeBindings. ].
	value isNil ifTrue: [
		bindings removeKey: key ifAbsent: []
	] ifFalse: [
		bindings at: key put: value
	].! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 11/19/2006 20:14'!
bindingOf: aString
	(bindings includesKey: aString)
		ifFalse: [bindings at: aString put: nil].
	^bindings associationAt: aString! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 7/13/2007 20:36'!
cr
	stream cr; flush.	! !

!REPL methodsFor: 'compiler' stamp: 'mvdg 11/20/2006 20:47'!
evaluate: aString
	[^ Compiler evaluate: aString for: self notifying: self logged: false ]
		ifError: [:err :recvr | ^ err, ' (', recvr name, ')']! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 11/7/2006 21:44'!
hasBindingOf: aVarName
	^ bindings includesKey: aVarName.! !

!REPL methodsFor: 'initialize-release' stamp: 'mvdg 11/19/2006 20:17'!
initializeBindings
	bindings := Dictionary new.
	bindings at: 'help' put: Help new.
	bindings at: 'self' put: self.
	bindings at: 'utils' put: REPLUtils new.
! !

!REPL methodsFor: 'initialize-release' stamp: 'mvdg 7/7/2007 17:19'!
initializeWithStream: aStream
	stream := aStream.
	stream prompt: ': '.
	self initializeBindings.! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 11/12/2006 21:58'!
isExit: aString
	| s |
	s := aString withBlanksTrimmed.
	^ (s = 'exit') or: [ s = 'quit' ].! !

!REPL methodsFor: 'compiler' stamp: 'avi 1/2/2004 10:38'!
notify: aString at: location in: sourceStream
	self selectFrom: location to: location + 2.
	self print: (aString allButLast: 3); cr.! !

!REPL methodsFor: 'stuff' stamp: 'avi 1/2/2004 11:13'!
print: aString
	stream nextPutAll: aString withInternetLineEndings; flush! !

!REPL methodsFor: 'stuff' stamp: 'avi 1/2/2004 10:38'!
select
  self error: 'Compilation error'! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 7/13/2007 20:47'!
selectFrom: aPos to: anotherPos
	"I'm called by the parser to print error messages. "
	stream prompt size + aPos - 1 timesRepeat: [stream nextPut: $ ].
	anotherPos - aPos + 1 timesRepeat: [stream nextPut: $^ ].
	stream nextPut: $ .
! !

!REPL methodsFor: 'compiler' stamp: 'avi 1/2/2004 10:38'!
selectionInterval
	^ 1 to: 1! !

!REPL methodsFor: 'stuff' stamp: 'mvdg 7/15/2007 21:09'!
start
	| line last |
	[
		line := stream nextLine.
		line ifNil: [ ^ self ].
		(self isExit: line) ifTrue: [^ self].
		last := self evaluate: line.
		(last == bindings) ifFalse: [self bind: 'last' to: last].
		[last printMultiLineOn: stream. self cr; cr.] 
			ifError: [ :e | self print: '  '; print: (e asString) ].
	] repeat.
	" A previous version of this method registered self to the Transcript. This can lead to a couple of problems, such as severe spammage from a mis-behaving process, or failing to save the image because saving can end up writing transcript messages to dead Sockets. "! !

!REPL methodsFor: 'stuff' stamp: 'avi 1/2/2004 10:38'!
text
	^ Text fromString: ''! !

!REPL methodsFor: 'stuff' stamp: 'avi 1/2/2004 10:38'!
update: aSymbol
	self print: Transcript contents! !

Object subclass: #REPLHinter
	instanceVariableNames: 'parser repl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!REPLHinter commentStamp: 'mvdg 11/7/2006 21:19' prior: 0!
I am used for tab completion by the InteractiveLineStream.!

!REPLHinter methodsFor: 'as yet unclassified' stamp: 'mvdg 11/23/2006 21:58'!
compilerTarget: c
	repl := c.! !

!REPLHinter methodsFor: 'as yet unclassified' stamp: 'mvdg 7/17/2007 21:44'!
completeLine: aLine
	| returnValue |
	self assert: [ aLine isKindOf: String ].
	returnValue := [ Compiler evaluate: aLine for: self notifying: self logged: false ] 
		ifError: [ :err :receiver | returnValue := err ].
	
	^ aLine, ' <tab completed>'.! !

!REPLHinter methodsFor: 'compiler' stamp: 'mvdg 11/23/2006 22:09'!
selectionInterval
	^ 1 to: 1.! !

Object subclass: #REPLTestingChecklist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!REPLTestingChecklist commentStamp: 'mvdg 7/18/2007 22:02' prior: 0!
This class exists only for this comment here :-).

- test to make sure that live connections aren't saved through image restarts.

- test all utils commands.

- test under load.

- Test starting / stopping image (check process list).

- Test extra long passwords (currently they fail?).

- test opening and then breaking a connection.!

Object subclass: #REPLUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'REPL'!

!REPLUtils methodsFor: 'as yet unclassified' stamp: 'mvdg 7/13/2007 21:30'!
allProcesses
	| all |
	all := Process allInstances reject: [ :each | each isTerminated ].
	^ ProcessList newFrom: (all sortBy: [ :a :b | a priority >= b priority ]).
		! !

!REPLUtils methodsFor: 'as yet unclassified' stamp: 'mvdg 7/15/2007 21:07'!
quitSqueak
	" The following is really wordy. "
	SmalltalkImage current snapshot: true andQuit: true.
	^ 'SmalltalkImage current snapshot: true andQuit: true.'! !

!REPLUtils methodsFor: 'as yet unclassified' stamp: 'mvdg 11/28/2006 23:13'!
rebootMorphic
	" Initialise a broken Morphic system. "
	[Project uiProcess terminate.
	Project newMorphic enter.] forkAt: (Processor userInterruptPriority).! !

!REPLUtils methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2006 21:09'!
suspendAllProcesses
	" Suspend all processes, except for the important ones. We make the assumption that non-important processes are running between priorities 20 (SystemBackgroundPriority) and 50 (UserInterruptPriority)."
	Process allInstances do: [ :each | 
		(((Processor systemBackgroundPriority) < (each priority)) 
			and: [(each priority) < (Processor userInterruptPriority)])
			ifTrue: [ each suspend ]. ].! !

!Symbol methodsFor: '*REPL-printing' stamp: 'mvdg 2/6/2007 12:24'!
printMultiLineOn: aStream
	aStream nextPut: $#.
	aStream nextPutAll: self.! !

!Dictionary methodsFor: '*REPL-printing' stamp: 'mvdg 7/13/2007 22:05'!
printMultiLineOn: aStream 
	aStream nextPut: $(.
	self size > 100
		ifTrue: [aStream nextPutAll: 'size '.
			self size printOn: aStream]
		ifFalse: [
			self keysSortedSafely
				do: [:key | aStream crtab; print: key;
					 nextPutAll: '->'.				
					 (self at: key) == self ifTrue: [ aStream nextPutAll: 'self' ] 
					     ifFalse: [aStream nextPutAll: (((self at: key) printStringLimitedTo: 54) 
								select: [ :each | each isTypeable ]).
					 ]]].
	aStream cr;nextPut: $)! !

!Character class methodsFor: '*REPL-more untypeable characters' stamp: 'mvdg 11/14/2006 22:02'!
beep
	^ self value: 7.! !

!Character class methodsFor: '*REPL-more untypeable characters' stamp: 'mvdg 11/14/2006 22:12'!
continue
	" On some systems, this continues a stopped process. "
	^ self value: 17.! !

!Character class methodsFor: '*REPL-more untypeable characters' stamp: 'mvdg 11/14/2006 21:54'!
interrupt
	^ self value: 3.! !

!Character class methodsFor: '*REPL-more untypeable characters' stamp: 'mvdg 11/14/2006 22:13'!
suspend
	^ self value: 19.! !

!Character methodsFor: '*REPL-shouldbeinSqueak' stamp: 'mvdg 11/12/2006 21:09'!
isTypeable
	" Is this character a printing character. I.e. is it not arrowUp / backspace / etc. "
	(value < 32) ifTrue: [ ^ false ].
	((value >= 127) and: [value < 160]) ifTrue: [ ^ false ].
	^ true.! !
